@@ -1,3 +1,3 @@
 from .components import *
 from .interactions import *
-from .slashcommands import *
+from .commands import *
